@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclFileName.c,v 1.40 2003/01/09 10:01:59 vincentdarley Exp $
+ * RCS: @(#) $Id: tclFileName.c,v 1.40.2.6 2004/01/13 17:26:42 dgp Exp $
  */
 
 #include "tclInt.h"
@@ -1380,49 +1380,32 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
 				 * with name after tilde substitution. */
 {
     Tcl_Obj *path = Tcl_NewStringObj(name, -1);
-    CONST char *result;
+    Tcl_Obj *transPtr;
 
     Tcl_IncrRefCount(path);
-    result = Tcl_FSGetTranslatedStringPath(interp, path);
-    if (result == NULL) {
+    transPtr = Tcl_FSGetTranslatedPath(interp, path);
+    if (transPtr == NULL) {
 	Tcl_DecrRefCount(path);
 	return NULL;
     }
+    
     Tcl_DStringInit(bufferPtr);
-    Tcl_DStringAppend(bufferPtr, result, -1);
+    Tcl_DStringAppend(bufferPtr, Tcl_GetString(transPtr), -1);
     Tcl_DecrRefCount(path);
-
+    Tcl_DecrRefCount(transPtr);
+    
     /*
      * Convert forward slashes to backslashes in Windows paths because
      * some system interfaces don't accept forward slashes.
      */
 
     if (tclPlatform == TCL_PLATFORM_WINDOWS) {
-#if defined(__CYGWIN__) && defined(__WIN32__)
-
-	extern int cygwin_conv_to_win32_path 
-	    _ANSI_ARGS_((CONST char *, char *));
-	char winbuf[MAX_PATH];
-
-	/*
-	 * In the Cygwin world, call conv_to_win32_path in order to use the
-	 * mount table to translate the file name into something Windows will
-	 * understand.  Take care when converting empty strings!
-	 */
-	if (Tcl_DStringLength(bufferPtr)) {
-	    cygwin_conv_to_win32_path(Tcl_DStringValue(bufferPtr), winbuf);
-	    Tcl_DStringFree(bufferPtr);
-	    Tcl_DStringAppend(bufferPtr, winbuf, -1);
-	}
-#else /* __CYGWIN__ && __WIN32__ */
-
 	register char *p;
 	for (p = Tcl_DStringValue(bufferPtr); *p != '\0'; p++) {
 	    if (*p == '/') {
 		*p = '\\';
 	    }
 	}
-#endif /* __CYGWIN__ && __WIN32__ */
     }
     return Tcl_DStringValue(bufferPtr);
 }
@@ -1602,7 +1585,6 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
     join = 0;
     dir = PATH_NONE;
     typePtr = NULL;
-    resultPtr = Tcl_GetObjResult(interp);
     for (i = 1; i < objc; i++) {
 	if (Tcl_GetIndexFromObj(interp, objv[i], options, "option", 0, &index)
 		!= TCL_OK) {
@@ -1628,14 +1610,14 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
 		break;
 	    case GLOB_DIR:				/* -dir */
 		if (i == (objc-1)) {
-		    Tcl_AppendToObj(resultPtr,
-			    "missing argument to \"-directory\"", -1);
+		    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+			    "missing argument to \"-directory\"", -1));
 		    return TCL_ERROR;
 		}
 		if (dir != PATH_NONE) {
-		    Tcl_AppendToObj(resultPtr,
+		    Tcl_SetObjResult(interp, Tcl_NewStringObj(
 			    "\"-directory\" cannot be used with \"-path\"",
-			    -1);
+			    -1));
 		    return TCL_ERROR;
 		}
 		dir = PATH_DIR;
@@ -1651,14 +1633,14 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
 		break;
 	    case GLOB_PATH:				/* -path */
 	        if (i == (objc-1)) {
-		    Tcl_AppendToObj(resultPtr,
-			    "missing argument to \"-path\"", -1);
+		    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+			    "missing argument to \"-path\"", -1));
 		    return TCL_ERROR;
 		}
 		if (dir != PATH_NONE) {
-		    Tcl_AppendToObj(resultPtr,
+		    Tcl_SetObjResult(interp, Tcl_NewStringObj(
 			    "\"-path\" cannot be used with \"-directory\"",
-			    -1);
+			    -1));
 		    return TCL_ERROR;
 		}
 		dir = PATH_GENERAL;
@@ -1667,8 +1649,8 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
 		break;
 	    case GLOB_TYPE:				/* -types */
 	        if (i == (objc-1)) {
-		    Tcl_AppendToObj(resultPtr,
-			    "missing argument to \"-types\"", -1);
+		    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+			    "missing argument to \"-types\"", -1));
 		    return TCL_ERROR;
 		}
 		typePtr = objv[i+1];
@@ -1680,7 +1662,6 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
 	    case GLOB_LAST:				/* -- */
 	        i++;
 		goto endOfForLoop;
-		break;
 	}
     }
     endOfForLoop:
@@ -1689,9 +1670,9 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
 	return TCL_ERROR;
     }
     if ((globFlags & TCL_GLOBMODE_TAILS) && (pathOrDir == NULL)) {
-	Tcl_AppendToObj(resultPtr,
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
 	  "\"-tails\" must be used with either \"-directory\" or \"-path\"",
-	  -1);
+	  -1));
 	return TCL_ERROR;
     }
     
@@ -1850,8 +1831,7 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
 		    }
 		}
 		/*
-		 * Error cases.  We re-get the interpreter's result,
-		 * just to be sure it hasn't changed, and we reset
+		 * Error cases.  We reset
 		 * the 'join' flag to zero, since we haven't yet
 		 * made use of it.
 		 */
@@ -1863,10 +1843,9 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
 		join = 0;
 		goto endOfGlob;
 		badMacTypesArg:
-		resultPtr = Tcl_GetObjResult(interp);
-		Tcl_AppendToObj(resultPtr,
+		Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		   "only one MacOS type or creator argument"
-		   " to \"-types\" allowed", -1);
+		   " to \"-types\" allowed", -1));
 		result = TCL_ERROR;
 		join = 0;
 		goto endOfGlob;
@@ -1882,11 +1861,6 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
      */
     objc -= i;
     objv += i;
-    /* 
-     * We re-retrieve this, in case it was changed in 
-     * the Tcl_ResetResult above 
-     */
-    resultPtr = Tcl_GetObjResult(interp);
     result = TCL_OK;
     if (join) {
 	if (dir != PATH_GENERAL) {
@@ -2330,8 +2304,19 @@ TclDoGlob(interp, separators, headPtr, tail, types)
     count = 0;
     name = tail;
     for (; *tail != '\0'; tail++) {
-	if ((*tail == '\\') && (strchr(separators, tail[1]) != NULL)) {
-	    tail++;
+	if (*tail == '\\') {
+	    /* 
+	     * If the first character is escaped, either we have a directory
+	     * separator, or we have any other character.  In the latter case
+	     * the rest of tail is a pattern, and we must break from the loop.
+	     * This is particularly important on Windows where '\' is both
+	     * the escaping character and a directory separator.
+	     */
+	    if (strchr(separators, tail[1]) != NULL) {
+		tail++;
+	    } else {
+		break;
+	    }
 	} else if (strchr(separators, *tail) == NULL) {
 	    break;
 	}
@@ -2378,25 +2363,6 @@ TclDoGlob(interp, separators, headPtr, tail, types)
 	     * element.  Add an extra slash if this is a UNC path.
 	     */
 
-#if defined(__CYGWIN__) && defined(__WIN32__)
-	    {
-
-	    extern int cygwin_conv_to_win32_path 
-	    	_ANSI_ARGS_((CONST char *, char *));
-	    char winbuf[MAX_PATH];
-
-	    /*
-	     * In the Cygwin world, call conv_to_win32_path in order to use
-	     * the mount table to translate the file name into something
-	     * Windows will understand.
-	     */
-	    cygwin_conv_to_win32_path(Tcl_DStringValue(headPtr), winbuf);
-	    Tcl_DStringFree(headPtr);
-	    Tcl_DStringAppend(headPtr, winbuf, -1);
-
-	    }
-#endif /* __CYGWIN__ && __WIN32__ */
-
 	    if (*name == ':') {
 		Tcl_DStringAppend(headPtr, ":", 1);
 		if (count > 1) {
@@ -2609,11 +2575,22 @@ TclDoGlob(interp, separators, headPtr, tail, types)
 		if (Tcl_DStringLength(headPtr) == 0) {
 		    if (((*name == '\\') && (name[1] == '/' || name[1] == '\\'))
 			    || (*name == '/')) {
-			Tcl_DStringAppend(headPtr, "\\", 1);
+			Tcl_DStringAppend(headPtr, "/", 1);
 		    } else {
 			Tcl_DStringAppend(headPtr, ".", 1);
 		    }
 		}
+#if defined(__CYGWIN__) && defined(__WIN32__)
+		{
+		extern int cygwin_conv_to_win32_path 
+		    _ANSI_ARGS_((CONST char *, char *));
+		char winbuf[MAX_PATH+1];
+
+		cygwin_conv_to_win32_path(Tcl_DStringValue(headPtr), winbuf);
+		Tcl_DStringFree(headPtr);
+		Tcl_DStringAppend(headPtr, winbuf, -1);
+		}
+#endif /* __CYGWIN__ && __WIN32__ */
 		/* 
 		 * Convert to forward slashes.  This is required to pass
 		 * some Tcl tests.  We should probably remove the conversions
