@@ -9,7 +9,7 @@
 # See the file "license.terms" for information on usage and
 # redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
-# RCS: @(#) $Id: http.tcl,v 1.43 2002/10/03 13:34:32 dkf Exp $
+# RCS: @(#) $Id: http.tcl,v 1.43.2.3 2003/10/02 23:07:33 dgp Exp $
 
 # Rough version history:
 # 1.0	Old http_get interface
@@ -25,7 +25,7 @@
 package require Tcl 8.2
 # keep this in sync with pkgIndex.tcl
 # and with the install directories in Makefiles
-package provide http 2.4.2
+package provide http 2.4.5
 
 namespace eval http {
     variable http
@@ -119,7 +119,7 @@ proc http::config {args} {
 	}
 	return $result
     }
-    regsub -all -- - $options {} options
+    set options [string map {- ""} $options]
     set pat ^-([join $options |])$
     if {[llength $args] == 1} {
 	set flag [lindex $args 0]
@@ -255,21 +255,28 @@ proc http::geturl { url args } {
 	status		""
 	http            ""
     }
+    # These flags have their types verified [Bug 811170]
+    array set type {
+	-binary		boolean
+	-blocksize	integer
+	-queryblocksize integer
+	-validate	boolean
+	-timeout	integer
+    }	
     set state(charset)	$defaultCharset
     set options {-binary -blocksize -channel -command -handler -headers \
 	    -progress -query -queryblocksize -querychannel -queryprogress\
 	    -validate -timeout -type}
     set usage [join $options ", "]
-    regsub -all -- - $options {} options
+    set options [string map {- ""} $options]
     set pat ^-([join $options |])$
     foreach {flag value} $args {
 	if {[regexp $pat $flag]} {
 	    # Validate numbers
-	    if {[info exists state($flag)] && \
-		    [string is integer -strict $state($flag)] && \
-		    ![string is integer -strict $value]} {
+	    if {[info exists type($flag)] && \
+		    ![string is $type($flag) -strict $value]} {
 		unset $token
-		return -code error "Bad value for $flag ($value), must be integer"
+		return -code error "Bad value for $flag ($value), must be $type($flag)"
 	    }
 	    set state($flag) $value
 	} else {
@@ -288,9 +295,11 @@ proc http::geturl { url args } {
     }
 
     # Validate URL, determine the server host and port, and check proxy case
+    # Recognize user:pass@host URLs also, although we do not do anything
+    # with that info yet.
 
-    if {![regexp -nocase {^(([^:]*)://)?([^/:]+)(:([0-9]+))?(/.*)?$} $url \
-	    x prefix proto host y port srvurl]} {
+    set exp {^(([^:]*)://)?([^@]+@)?([^/:]+)(:([0-9]+))?(/.*)?$}
+    if {![regexp -nocase $exp $url x prefix proto user host y port srvurl]} {
 	unset $token
 	return -code error "Unsupported URL: $url"
     }
@@ -414,7 +423,7 @@ proc http::geturl { url args } {
 	}
 	puts $s "User-Agent: $http(-useragent)"
 	foreach {key value} $state(-headers) {
-	    regsub -all \[\n\r\]  $value {} value
+	    set value [string map [list \n "" \r ""] $value]
 	    set key [string trim $key]
 	    if {[string equal $key "Content-Length"]} {
 		set contDone 1
@@ -678,8 +687,9 @@ proc http::Event {token} {
 	} elseif {$n == 0} {
 	    variable encodings
 	    set state(state) body
-	    if {$state(-binary) || ![regexp -nocase ^text $state(type)] || \
-		    [regexp gzip|compress $state(coding)]} {
+	    if {$state(-binary) || ![string match -nocase text* $state(type)]
+		    || [string match *gzip* $state(coding)]
+		    || [string match *compress* $state(coding)]} {
 		# Turn off conversions for non-text data
 		fconfigure $s -translation binary
 		if {[info exists state(-channel)]} {
@@ -716,7 +726,7 @@ proc http::Event {token} {
 	    }
 	    if {[regexp -nocase {^([^:]+):(.+)$} $line x key value]} {
 		lappend state(meta) $key [string trim $value]
-	    } elseif {[regexp ^HTTP $line]} {
+	    } elseif {[string match HTTP* $line]} {
 		set state(http) $line
 	    }
 	}
